<?php

/**
 * @file
 * Contains \Drupal\slick_media\Plugin\Field\FieldFormatter\SlickMediaFormatter.
 */

namespace Drupal\slick_media\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\slick\Plugin\Field\FieldFormatter\SlickEntityReferenceFormatterBase;
use Drupal\slick\SlickFormatterInterface;
use Drupal\media_entity_embeddable_video\VideoProviderManager;

/**
 * Plugin implementation of the 'slick media entity' formatter.
 *
 * @FieldFormatter(
 *   id = "slick_media",
 *   label = @Translation("Slick media"),
 *   description = @Translation("Display the referenced entities as a Slick carousel."),
 *   field_types = {"entity_reference"},
 *   quickedit = {"editor" = "disabled"}
 * )
 */
class SlickMediaFormatter extends SlickEntityReferenceFormatterBase implements ContainerFactoryPluginInterface {

  /**
   * The video provider manager.
   *
   * @var \Drupal\media_entity_embeddable_video\VideoProviderManager
   */
  protected $videoManager;

  /**
   * Constructs a SlickMediaFormatter instance.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, LoggerChannelFactoryInterface $logger_factory, SlickFormatterInterface $formatter, VideoProviderManager $video_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $logger_factory, $formatter);
    $this->videoManager = $video_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('logger.factory'),
      $container->get('slick.formatter'),
      $container->get('plugin.manager.media_entity_embeddable_video.provider')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $entities = $this->getEntitiesToView($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($entities)) {
      return [];
    }

    $build          = $this->formatter->buildSettings($items, $langcode, $this->getSettings());
    $settings       = $build['settings'];
    $build['items'] = $this->buildElements($entities, $langcode, $settings);
    $build['thumb'] = !$settings['nav'] ? [] : ['items' => $this->buildElements($entities, $langcode, $settings, TRUE)];

    return $this->manager()->build($build);
  }

  /**
   * {@inheritdoc}
   */
  public function buildMedia($entity, $langcode, $slide, $delta) {
    $media = parent::buildMedia($entity, $langcode, $slide, $delta);

    $source_field[$entity->bundle()] = $entity->getType()->getConfiguration()['source_field'];

    $media['source_field'] = $source_field[$entity->bundle()];
    if (!empty($media['source_field'])) {
      $media_url = $this->getFieldString($entity, $media['source_field'], $langcode);

      /** @var \Drupal\media_entity_embeddable_video\VideoProviderInterface $provider */
      if ($media_url && $provider = $this->videoManager->getProviderByEmbedCode($media_url)) {
        $media['url']           = $provider->render()['#attributes']['src'];
        $media['scheme']        = $provider->id();
        $media['thumbnail_uri'] = $provider->thumbnailURI();
        // @todo get 'type' independent from bundle names: image, video, audio.
        $media['type']          = 'video';
        $media['video_id']      = $provider->videoId();
      }
    }

    return $media;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    $storage = $field_definition->getFieldStorageDefinition();
    return $storage->isMultiple() && $storage->getSetting('target_type') === 'media';
  }

}
